package com.loafle.overflow.container.service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.loafle.overflow.core.annotation.RPCService;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.interfaces.Service;
import com.loafle.overflow.model.sensorconfig.SensorConfig;
import com.loafle.overflow.service.container.SensorConfigService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * SensorConfigService
 */
@RPCService("SensorConfigService")
public class ContainerSensorConfigService implements Service, SensorConfigService {
  private static final Logger logger = LoggerFactory.getLogger(ContainerSensorConfigService.class);

  private Map<String, SensorConfig> sensorConfigs;

  @Override
  public void initService() throws Exception {
    this.sensorConfigs = new HashMap<>();
  }

  @Override
  public void startService() throws Exception {

  }

  @Override
  public void stopService() {

  }

  @Override
  public void destroyService() {

  }

  SensorConfig getSensorConfig(String configID) {
    return this.sensorConfigs.get(configID);
  }

  @Override
  public void InitConfig(List<SensorConfig> sensorConfigs) throws OverflowException {
    if (null == sensorConfigs || 0 == sensorConfigs.size()) {
      return;
    }

    for (SensorConfig sensorConfig : sensorConfigs) {
      this.sensorConfigs.put(sensorConfig.getConfigID(), sensorConfig);
    }
    logger.debug("Sensor configs[%d] were added", sensorConfigs.size());
  }

  @Override
  public void AddConfig(SensorConfig sensorConfig) throws OverflowException {
    if (null == sensorConfig) {
      throw new OverflowException("Sensor config is not valid", null);
    }

    String configID = sensorConfig.getConfigID();
    if (this.sensorConfigs.containsKey(configID)) {
      throw new OverflowException(String.format("Sensor config[%s] is exist already", configID), null);
    }

    this.sensorConfigs.put(configID, sensorConfig);

    logger.debug("Sensor config[%s] was added", configID);
  }

  @Override
  public void UpdateConfig(SensorConfig sensorConfig) throws OverflowException {
    if (null == sensorConfig) {
      throw new OverflowException("Sensor config is not valid", null);
    }

    String configID = sensorConfig.getConfigID();
    if (!this.sensorConfigs.containsKey(configID)) {
      throw new OverflowException(String.format("Sensor config[%s] is not exist", configID), null);
    }

    this.sensorConfigs.remove(configID);
    this.sensorConfigs.put(configID, sensorConfig);

    logger.debug("Sensor config[%s] was updated", configID);
  }

  @Override
  public void RemoveConfig(String configID) throws OverflowException {
    if (!this.sensorConfigs.containsKey(configID)) {
      throw new OverflowException(String.format("Sensor config[%s] is not exist", configID), null);
    }

    this.sensorConfigs.remove(configID);

    logger.debug("Sensor config[%s] was removed", configID);
  }
}